import os
import os_tornado
from tests.test_commands.cmd import run


def test_command_startproject(tmpdir):
    paths = """
    temp/manager.py
    temp/app/settings.py
    temp/app/__init__.py
    temp/app/extensions/__init__.py
    temp/app/extensions/example_extension.py
    temp/app/request_handlers/__init__.py
    temp/app/request_handlers/example_handler.py
    """
    run('startproject temp %s' % tmpdir.strpath)
    for p in paths.split():
        assert tmpdir.join(p.strip()).exists()


def test_command_list():
    env = os.environ.copy()
    env["OS_TORNADO_SETTINGS_MODULE"] = 'tests.test_commands.settings'
    result, _ = run('list', env=env)
    assert '[ext/req/all]' in result
    result, _ = run('list ext', env=env)
    assert 'TempExtension' in result
    result, _ = run('list req', env=env)
    assert 'TempHandler' in result
    result, _ = run('list all', env=env)
    assert 'TempExtension' in result
    assert 'TempHandler' in result


def test_command_version():
    result, _ = run('version')
    assert os_tornado.__version__ in result


def test_unkown_cmd():
    cmd = 'unknown'
    result, _ = run(cmd)
    assert 'Unknown command: %s' % cmd in result


def test_cmdline():
    result, _ = run()
    for cmd in ['list', 'runserver', 'startproject', 'version']:
        assert cmd in result



"""
This module contains the default values used by os-torado.
"""

LOG_ENABLED = True
LOG_LEVEL = 'DEBUG'
LOG_FORMAT = '[%(asctime)s] [%(name)s] [%(levelname)s] %(message)s'
LOG_DATEFORMAT = '%Y-%m-%d %H:%M:%S'

HTTP_PORT = 8080
BIND_ADDRESS = ''
DEFAULT_HOST = ''



"""Request Handler Manager"""
import logging
import copy
import tornado.web
from os_tornado.utils.module_utils import load_class


class RequestHandlerManager(object):
    """Request Handler Manager"""

    def __init__(self, manager):
        self._manager = manager
        self._handlers = []
        self._logger = logging.getLogger('RequestHanlderManager')
        self._loaded = False

    def load(self):
        if self._loaded:
            return
        handlers = self._manager.settings['REQUEST_HANDLERS']
        if not handlers:
            return
        for h_settings in handlers:
            h_settings = copy.deepcopy(h_settings)
            handler_cls = self._load_cls(h_settings)
            if not handler_cls:
                continue
            pattern = h_settings['pattern']

            self._handlers.append(
                (pattern, handler_cls, h_settings))
            self._logger.info(
                '[LOAD] [SUCC] %s %s.%s',
                pattern, handler_cls.__module__, handler_cls.__name__)
        self._loaded = True

    def _load_cls(self, handler_settings):
        handler_class = None
        try:
            handler_settings['pattern']
            _cls = load_class(
                handler_settings['handler_class'], tornado.web.RequestHandler)
            if _cls:
                handler_class = _cls
            else:
                self._logger.warn('[LOAD] [FAIL] invalid: %s',
                                  handler_settings['handler_class'])
        except Exception as e:
            self._logger.error('[LOAD] [FAIL] %s', e)
        return handler_class

    def iter_handlers(self):
        return iter(self._handlers)

    def get_all_handlers(self):
        return self._handlers



"""
Command line
"""
from __future__ import print_function
import sys
import os
import optparse
import os_tornado
from os_tornado.exceptions import UsageError
from os_tornado.settings import Settings
from os_tornado.commands import Command
from os_tornado.utils.module_utils import iter_classes

ENVVAR = 'OS_TORNADO_SETTINGS_MODULE'


def _get_settings():
    settings = Settings()
    settings.update_from_module('os_tornado.settings.default_settings')
    if ENVVAR in os.environ:
        settings.update_from_module(os.environ.get(ENVVAR))
    return settings


def _get_commands_from_module(module, settings):
    d = {}
    for cmd in iter_classes(module, Command):
        cmd_name = cmd.__module__.split('.')[-1]
        d[cmd_name] = cmd(settings)
    return d


def _get_commands_dict(settings):
    cmds = _get_commands_from_module('os_tornado.commands', settings)
    cmds_module = settings['COMMANDS_MODULE']
    if cmds_module:
        cmds.update(_get_commands_from_module(cmds_module, settings))
    return cmds


def _pop_command_name(argv):
    for arg in argv[1:]:
        if not arg.startswith('-'):
            return arg


def _print_header(settings):
    print("os-tornado %s\n" % os_tornado.__version__)


def _print_commands(settings):
    _print_header(settings)
    print("Usage:")
    print("  %s <command> [options] [args]\n" % sys.argv[0])
    print("Available commands:")
    cmds = _get_commands_dict(settings)
    for cmdname, cmdclass in sorted(cmds.items()):
        print("  %-18s %s" % (cmdname, cmdclass.short_desc()))
    print()
    print('Use "%s <command> -h" to see more info about a command' %
          sys.argv[0])


def _print_unknown_command(settings, cmd_name):
    _print_header(settings)
    print("Unknown command: %s\n" % cmd_name)
    print('Use "%s" to see available commands' % sys.argv[0])


def _run_print_help(parser, func, *a, **kw):
    try:
        func(*a, **kw)
    except UsageError as e:
        if str(e):
            parser.error(str(e))
        if e.print_help:
            parser.print_help()
        sys.exit(2)


def execute(argv=None):
    argv = argv or sys.argv
    settings = _get_settings()
    cmds = _get_commands_dict(settings)
    cmd_name = _pop_command_name(argv)
    parser = optparse.OptionParser(formatter=optparse.TitledHelpFormatter(),
                                   conflict_handler='resolve')
    if not cmd_name:
        _print_commands(settings)
        sys.exit(0)
    elif cmd_name not in cmds:
        _print_unknown_command(settings, cmd_name)
        sys.exit(2)
    cmd = cmds[cmd_name]
    parser.usage = "%s %s %s" % (argv[0], cmd_name, cmd.syntax())
    parser.description = cmd.long_desc()
    cmd.add_options(parser)
    opts, args = parser.parse_args(args=argv[2:])
    _run_print_help(parser, cmd.process_options, args, opts)
    _run_print_help(parser, cmd.run, args, opts)


if __name__ == '__main__':
    execute()



import logging
from logging.config import dictConfig


DEFAULT_LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'incremental': True,
}


def configure_logging(settings):
    dictConfig(DEFAULT_LOGGING)
    if settings["LOG_ENABLED"]:
        handler = logging.StreamHandler()
    else:
        handler = logging.NullHandler()

    formatter = logging.Formatter(
        fmt=settings['LOG_FORMAT'],
        datefmt=settings['LOG_DATEFORMAT'])
    logging.root.setLevel(logging.NOTSET)
    handler.setFormatter(formatter)
    handler.setLevel(settings['LOG_LEVEL'])
    logging.root.addHandler(handler)


import signal
import shlex
import sys
import os
import time
import subprocess
from os_tornado.utils.signal_utils import install_shutdown_handlers


_IT_WORKS = 123


def subproc():
    if os.getenv('COVERAGE_PROCESS_START'):
        import coverage
        coverage.process_startup()

    def handler(signum, frame):
        sys.exit(_IT_WORKS)
    install_shutdown_handlers(handler)
    print('start')
    count = 0
    while count < 5:
        time.sleep(1)
        count += 1


def test_install_shutdown_handlers():
    test_env = os.environ.copy()
    if os.getenv('COVERAGE'):
        test_env['COVERAGE_PROCESS_START'] = '.coveragerc'

    proc = subprocess.Popen(shlex.split('python -u %s' %
                                        os.path.abspath(__file__)),
                            stdout=subprocess.PIPE, env=test_env)
    while 'start' not in proc.stdout.readline():
        time.sleep(0.1)
    proc.send_signal(signal.SIGTERM)
    proc.communicate()
    assert _IT_WORKS == proc.returncode


if __name__ == "__main__":
    subproc()



"""The list command"""
from __future__ import print_function
from os_tornado.commands import Command
from os_tornado.component_manager import ComponentManager
from os_tornado.exceptions import UsageError


_TYPE = set(['ext', 'req', 'all'])


class ListCommand(Command):
    """Command for listing extensions and request handlers."""

    def syntax(self):
        return "[ext/req/all]"

    def short_desc(self):
        return "List available extensions and request handlers."

    def _show_ext(self, manager):
        exts = self.settings['EXTENSIONS']
        if not exts:
            print('No available extensions')
            return
        manager.load_extensions()
        print('Extensions:')
        for ext in manager.iter_extensions():
            print("  %-18s %s" %
                  (ext.name, '.'.join((ext.__module__, ext.__class__.__name__))))

    def _show_req(self, manager):
        handlers = self.settings['REQUEST_HANDLERS']
        if not handlers:
            print('No available request handlers')
            return
        manager.load_request_handlers()
        print('RequestHandlers:')
        for pattern, handler, _ in manager.iter_request_handlers():
            print("  %-18s %s" %
                  (pattern, '.'.join((handler.__module__, handler.__name__))))

    def run(self, args, opts):
        cm = _TYPE.intersection(args)
        if len(cm) <= 0:
            raise UsageError()
        if 'all' in cm:
            cm.remove('all')
            cm.update(['ext', 'req'])
        method = {'ext': self._show_ext, 'req': self._show_req}
        manager = ComponentManager(self.settings)
        for m in cm:
            method[m](manager)
            print('\n')



import re
import pytest
import logging
from os_tornado.settings import Settings, default_settings
from os_tornado.log import configure_logging


@pytest.fixture(scope="function")
def settings():
    settings = Settings()
    settings.update_from_module(default_settings)
    return settings


def test_log_format(settings, capsys):
    settings['LOG_FORMAT'] = '[%(asctime)s] [%(levelname)s] [%(name)s] %(message)s'
    configure_logging(settings)
    logger = logging.getLogger('test')
    log_string = 'test logging'
    logger.info(log_string)
    _, err = capsys.readouterr()
    regex = re.compile(r'''
    \[\d{4}-\d{2}-\d{2}\s\d{2}:\d{2}:\d{2}\]
    \s\[INFO\]
    \s\[test\]
    \stest\slogging
    ''', re.X)
    assert regex.match(err)


def test_log_level(settings, capsys):
    settings['LOG_LEVEL'] = 'INFO'
    configure_logging(settings)
    logger = logging.getLogger('test')
    log_string = 'test logging'
    logger.info(log_string)
    _, err = capsys.readouterr()
    assert log_string in err
    logger.debug(log_string)
    _, err = capsys.readouterr()
    assert log_string not in err


def test_base_log(settings, capsys):
    configure_logging(settings)
    logger = logging.getLogger('test')
    log_string = 'test logging'
    logger.info(log_string)
    _, err = capsys.readouterr()
    assert log_string in err


def test_disable_log(settings, capsys):
    settings['LOG_ENABLED'] = False
    configure_logging(settings)
    logger = logging.getLogger('test')
    log_string = 'test logging'
    logger.info(log_string)
    _, err = capsys.readouterr()
    assert log_string not in err



import shlex
import os
import sys
import subprocess
from os_tornado.cmdline import execute


def run(cmdline='', env=None, ** kwargs):
    if env is None:
        env = os.environ.copy()
    if env.get('COVERAGE', None) is not None:
        env['COVERAGE_PROCESS_START'] = os.path.abspath('.coveragerc')
    encoding = getattr(sys.stdout, 'encoding') or 'utf-8'
    cmd = 'python -u %s %s' % (os.path.abspath(__file__), cmdline)
    proc = subprocess.Popen(shlex.split(cmd),
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE,
                            env=env,
                            cwd=os.getcwd(),
                            **kwargs)
    stdout, stderr = proc.communicate()
    return stdout.decode(encoding), stderr.decode(encoding)


if __name__ == "__main__":
    sys.path.insert(0, os.getcwd())
    if os.getenv('COVERAGE_PROCESS_START'):
        import coverage
        coverage.process_startup()
    execute()



import pytest
from os_tornado.settings import Settings


@pytest.fixture(scope="function")
def settings():
    return Settings()


def test_get_tornado_app_settings(settings):
    from os_tornado.settings import TORNADO_APP_SETTINGS_PREFIX
    from os_tornado.settings import get_tornado_app_settings
    key = TORNADO_APP_SETTINGS_PREFIX + "COOKIE_SECRET"
    settings[key] = 'random_string'
    assert get_tornado_app_settings(settings)['cookie_secret'] == settings[key]


def test_iter_overridden_settings(settings):
    from os_tornado.settings import iter_overridden_settings, default_settings
    settings.update_from_module(default_settings)
    count = 0
    for key, value in iter_overridden_settings(settings):
        count += 1
    assert count == 0
    settings["HTTP_PORT"] = 8000
    settings["LOG_ENABLE"] = True
    count = 0
    for key, value in iter_overridden_settings(settings):
        count += 1
    assert count == 1


def test_iter_default_settings(settings):
    from os_tornado.settings import default_settings
    settings.update_from_module(default_settings)
    for key in settings:
        assert getattr(default_settings, key) == settings[key]


TEST_KEY1 = 1
TEST_KEY2 = '2'
TEST_KEY3 = True
test_Key4 = [1, 2, 3]


def test_update_from_module(settings):
    settings.update_from_module('tests.test_settings')
    assert settings['TEST_KEY1'] == 1
    assert settings.get('TEST_KEY2') == '2'
    assert settings['TEST_KEY3']
    assert len(settings) == 3


def test_get(settings):
    test_cases = [
        # value , settings.get_x, expected, Exception
        (True, settings.get_bool, True, None),
        (1, settings.get_bool, True, None),
        (0, settings.get_bool, False, None),
        ('0', settings.get_bool, False, None),
        (None, settings.get_bool, False, TypeError),
        (1, settings.get_int, 1, None),
        ('1', settings.get_int, 1, None),
        ([1, 2, 3], settings.get_list, [1, 2, 3], None),
        ('1, 2,3', settings.get_list, ['1', '2', '3'], None),
        (1, settings.get_list, ['1', '2', '3'], TypeError),
        ({1: 1}, settings.get_dict, {1: 1}, None),
        ('{"1": 1}', settings.get_dict, {"1": 1}, None),
        (1.2, settings.get_float, 1.2, None),
        ("1.2", settings.get_float, 1.2, None),

    ]

    count = 0
    for value, get_method, expected, exception in test_cases:
        count += 1
        key = 'KEY%d' % count
        settings[key] = value
        if exception:
            with pytest.raises(exception):
                get_method(key)
        else:
            assert get_method(key) == expected


def test_set_item(settings):
    settings[1] = 1
    settings[1] = 2
    assert settings[1] == 2


def test_contain(settings):
    assert 1 not in settings
    settings[1] = 1
    assert 1 in settings


def test_len(settings):
    assert len(settings) == 0
    settings[1] = 1
    assert len(settings) == 1


def test_get_item(settings):
    settings[1] = 1
    assert settings.get(1) == 1
    assert settings.get(2) is None
    assert settings.get(2, 1) == 1


def test_delete_item(settings):
    settings[1] = 1
    assert settings[1] == 1
    del settings[1]
    assert settings[1] is None


def test_init_settings():
    d = {1: 1, 2: 2}
    s = Settings(d)
    assert d == s.attributes


def test_freeze(settings):
    settings[1] = 1
    settings.freeze()
    with pytest.raises(TypeError):
        settings[2] = 2
    with pytest.raises(TypeError):
        del settings[2]



import logging
import tornado
from os_tornado.log import configure_logging
from os_tornado.settings import get_tornado_app_settings, get_tornado_server_settings
from os_tornado.utils.signal_utils import install_shutdown_handlers


class Runner(object):
    def __init__(self, manager):
        self._manager = manager
        self._logger = logging.getLogger('Runner')
        install_shutdown_handlers(self._cleanup_and_stop)

    @property
    def settings(self):
        return self._manager.settings

    def _cleanup_and_stop(self, signum, frame):
        self._logger.debug('Recieve stop signal %d', signum)
        self._logger.info('[CLEANUP] Extensions')
        self._manager.cleanup_extensions()
        tornado.ioloop.IOLoop.current().add_callback_from_signal(
            self._do_stop, signum, frame)

    def _do_stop(self, signum, frame):
        self._logger.info('stop')
        tornado.ioloop.IOLoop.current().stop()

    def run(self):
        self.settings.freeze()
        configure_logging(self.settings)
        self._logger.info('[LOAD] extensions')
        self._manager.load_extensions()
        self._logger.info('[SETUP] extensions')
        self._manager.setup_extensions()

        if self.settings["HTTP_PORT"]:
            self._logger.info('[LOAD] request handlers')
            self._manager.load_request_handlers()
            app = tornado.web.Application(
                self._manager.get_all_request_handlers(),
                default_host=self.settings.get('DEFAULT_HOST', ''),
                transforms=None,
                **get_tornado_app_settings(self.settings))
            app.manager = self._manager
            port = self.settings.get_int("HTTP_PORT")
            bind_address = self.settings.get('BIND_ADDRESS', '')
            server_settings = get_tornado_server_settings(self.settings)
            app.listen(port, bind_address, **server_settings)
            self._logger.info('listen port %d', port)
        else:
            self._logger.warn('no http interface, HTTP_PORT: %s',
                              str(self.settings['HTTP_PORT']))
        self._logger.info('[RUN] extensions')
        self._manager.run_extensions()

        tornado.ioloop.IOLoop.current().start()



"""Base class for Scrapy commands"""
from optparse import OptionGroup

from os_tornado.exceptions import UsageError


def arglist_to_dict(arglist):
    """Convert a list of arguments like ['arg1=val1', 'arg2=val2', ...] to a
    dict
    """
    return dict(x.split('=', 1) for x in arglist)


class Command(object):

    def __init__(self, settings=None):
        self.settings = settings

    def syntax(self):
        """
        Command syntax (preferably one-line). Do not include command name.
        """
        return ""

    def short_desc(self):
        """
        A short description of the command
        """
        return ""

    def long_desc(self):
        """A long description of the command. Return short description when not
        available. It cannot contain newlines, since contents will be formatted
        by optparser which removes newlines and wraps text.
        """
        return self.short_desc()

    def help(self):
        """An extensive help for the command. It will be shown when using the
        "help" command. It can contain newlines, since not post-formatting will
        be applied to its contents.
        """
        return self.long_desc()

    def add_options(self, parser):
        """
        Populate option parse with options available for this command
        """
        group = OptionGroup(parser, "Global Options")
        group.add_option("-L", "--loglevel", metavar="LEVEL", default=None,
                         help="log level (default: %s)" % self.settings['LOG_LEVEL'])
        group.add_option("-s", "--settings", action="append", default=[], metavar="NAME=VALUE",
                         help="set/override setting (may be repeated)")

        parser.add_option_group(group)

    def process_options(self, args, opts):
        try:
            self.settings.update(arglist_to_dict(opts.settings))
        except ValueError:
            raise UsageError(
                "Invalid -s value, use -s NAME=VALUE", print_help=False)

        if opts.loglevel:
            self.settings.set('LOG_LEVEL', opts.loglevel)

    def run(self, args, opts):
        """
        Entry point for running commands
        """
        raise NotImplementedError



from __future__ import print_function
from os_tornado.commands import Command
import os_tornado


class VersionCommand(Command):
    """Command for showing version"""

    def short_desc(self):
        return 'Print os-tornado version.'

    def run(self, args, opts):
        print("os-tornado %s" % os_tornado.__version__)



"""Component Manager"""
from os_tornado.extension import ExtensionManager
from os_tornado.request_handler_manager import RequestHandlerManager


class ComponentManager(object):
    """Component Manager

    Access to extensions and request handlers
    """
    def __init__(self, settings):
        self._settings = settings
        self._ext_manager = ExtensionManager(self)
        self._request_handler_manager = RequestHandlerManager(self)

    def load_extensions(self):
        self._ext_manager.load()

    def load_request_handlers(self):
        self._request_handler_manager.load()

    def get_extension(self, name):
        return self._ext_manager.get_extension(name)

    @property
    def settings(self):
        return self._settings

    def iter_extensions(self):
        return self._ext_manager.iter_extensions()

    def iter_request_handlers(self):
        return self._request_handler_manager.iter_handlers()

    def get_all_request_handlers(self):
        return self._request_handler_manager.get_all_handlers()

    def setup_extensions(self):
        self._ext_manager.setup()

    def cleanup_extensions(self):
        self._ext_manager.cleanup()

    def run_extensions(self):
        self._ext_manager.run()



"""The runserver command"""
from __future__ import print_function
from os_tornado.commands import Command
from os_tornado.component_manager import ComponentManager
from os_tornado.runner import Runner


class RunserverCommand(Command):
    """Command for starting server"""

    def short_desc(self):
        return "run server."

    def add_options(self, parser):
        super(RunserverCommand, self).add_options(parser)
        default_port = self.settings["HTTP_PORT"]
        parser.add_option("-p", "--port", action="store", dest="http_port",
                          type=int, default=default_port,
                          help="set http port (default: %s)" % str(default_port))

    def process_options(self, args, opts):
        if opts.http_port:
            self.settings["HTTP_PORT"] = opts.http_port

    def run(self, args, opts):
        manager = ComponentManager(self.settings)
        runner = Runner(manager)
        runner.run()



import sys
__all__ = ['__version__', 'version_info']

import pkgutil
__version__ = pkgutil.get_data(__package__, 'VERSION').decode('ascii').strip()
version_info = tuple(int(v) if v.isdigit() else v
                     for v in __version__.split('.'))
del pkgutil

if sys.version_info < (2, 7):
    print("os-tornado %s requires Python 2.7" % __version__)
    sys.exit(1)

del sys


"""
Extension
"""
import collections
import logging
from os_tornado.utils.module_utils import load_class


class ExtensionManager(object):
    def __init__(self, manager):
        self._manager = manager
        self._exts = collections.OrderedDict()
        self._logger = logging.getLogger('ExtManager')
        self._loaded = False

    def load(self):
        if self._loaded:
            return
        exts = self._manager.settings['EXTENSIONS']
        if not exts:
            return
        for ext_settings in exts:
            ext = self._load_extension(ext_settings)
            c_path = ext_settings['extension_class']
            if not ext:
                continue
            if ext.name not in self._exts:
                self._exts[ext.name] = ext
                self._logger.info('[LOAD] [SUCC] %s %s.%s',
                                  ext.name, ext.__module__, ext.__class__.__name__)
            else:
                self._logger.warn(
                    '[LOAD] [SKIP] already exist: %s, %s', ext.name, c_path)
        self._loaded = True

    def _load_extension(self, ext_settings):
        ext = None
        try:
            _cls = load_class(ext_settings['extension_class'], Extension)
            if _cls:
                ext = _cls(self._manager, ext_settings)
            else:
                self._logger.warn('[LOAD] [FAIL] invalid: %s',
                                  ext_settings['extension_class'])
        except Exception, e:
            self._logger.error('[LOAD] [FAIL] %s', e)
        return ext

    def iter_extensions(self):
        for name in self._exts:
            yield self._exts[name]

    def setup(self):
        for ext in self._exts.values():
            try:
                ext.setup()
            except Exception as e:
                self._logger.error('[SETUP] %s %s' % (ext.name, e))

    def cleanup(self):
        for ext in reversed(self._exts.values()):
            try:
                ext.cleanup()
            except Exception as e:
                self._logger.error('[CLEANUP] %s %s' % (ext.name, e))

    def run(self):
        for ext in self._exts.values():
            try:
                ext.run()
            except Exception as e:
                self._logger.error('[RUN] %s %s' % (ext.name, e))

    def get_extension(self, name):
        return self._exts.get(name, None)


class Extension(object):
    def __init__(self, manager, ext_settings):
        self._manager = manager
        self._ext_settings = ext_settings
        self._name = ext_settings.get('name', self.__class__.__name__).strip()
        self._logger = logging.getLogger('Ext/%s' % self._name)

    @property
    def name(self):
        return self._name

    @property
    def ext_settings(self):
        return self._ext_settings

    @property
    def settings(self):
        return self._manager.settigns

    def run(self):
        pass

    def setup(self):
        pass

    def cleanup(self):
        pass



"""
Core Exceptions
"""


class UsageError(Exception):
    """To indicate a command-line usage error"""

    def __init__(self, *a, **kw):
        self.print_help = kw.pop('print_help', True)
        super(UsageError, self).__init__(*a, **kw)



EXTENSIONS = [
    {
        "name": "Temp",
        "extension_class": "tests.test_commands.temp_extension.TempExtension",
        "key1": "value1",
    }
]

PORT = 9999
REQUEST_HANDLERS = [
    {
        "pattern": r"/",
        "handler_class": "tests.test_commands.temp_handler.TempHandler",
        "key1": "value1"
    }
]



"""
Settings
"""

from importlib import import_module
import six
import json
from collections import MutableMapping
from . import default_settings

TORNADO_APP_SETTINGS_PREFIX = "TORNADO_APP_SETTINGS_"
TORNADO_SERVER_SETTINGS_PREFIX = "TORNADO_SERVER_SETTINGS_"


class Settings(MutableMapping):

    def __init__(self, attributes=None):
        self.frozen = False
        self.attributes = {}
        self.update(attributes)

    def __getitem__(self, name):
        if name not in self:
            return None
        return self.attributes[name]

    def __contains__(self, name):
        return name in self.attributes

    def __setitem__(self, name, value):
        self._assert_mutability()
        self.attributes[name] = value

    def __delitem__(self, name):
        self._assert_mutability()
        del self.attributes[name]

    def __iter__(self):
        return iter(self.attributes)

    def __len__(self):
        return len(self.attributes)

    def get(self, name, default=None):
        """
        Get a setting value without affecting its original type.

        :param name: the setting name
        :type name: string

        :param default: the value to return if no setting is found
        :type default: any
        """
        return self[name] if name in self else default

    def _assert_mutability(self):
        if self.frozen:
            raise TypeError("Trying to modify an immutable Settings object")

    def get_bool(self, name, default=False):
        """
        Get a setting value as a boolean.

        ``1``, ``'1'``, `True`` and ``'True'`` return ``True``,
        while ``0``, ``'0'``, ``False``, ``'False'`` and ``None`` return ``False``.

        For example, settings populated through environment variables set to
        ``'0'`` will return ``False`` when using this method.

        :param name: the setting name
        :type name: string

        :param default: the value to return if no setting is found
        :type default: any
        """
        got = self.get(name, default)
        try:
            return bool(int(got))
        except ValueError:
            if got in ("True", "true"):
                return True
            if got in ("False", "false"):
                return False
            raise ValueError("Supported values for boolean settings "
                             "are 0/1, True/False, '0'/'1', "
                             "'True'/'False' and 'true'/'false'")

    def get_int(self, name, default=0):
        """
        Get a setting value as an int.

        :param name: the setting name
        :type name: string

        :param default: the value to return if no setting is found
        :type default: any
        """
        return int(self.get(name, default))

    def get_list(self, name, default=None):
        """
        Get a setting value as a list. If the setting original type is a list, a
        copy of it will be returned. If it's a string it will be split by ",".

        For example, settings populated through environment variables set to
        ``'one,two'`` will return a list ['one', 'two'] when using this method.

        :param name: the setting name
        :type name: string

        :param default: the value to return if no setting is found
        :type default: any
        """
        value = self.get(name, default or [])
        if isinstance(value, six.string_types):
            value = value.split(',')
            value = [v.strip() for v in value]
        return list(value)

    def get_dict(self, name, default=None):
        """
        Get a setting value as a dictionary. If the setting original type is a
        dictionary, a copy of it will be returned. If it is a string it will be
        evaluated as a JSON dictionary. In the case that it is a
        :class:`~os_tornado.settings.Settings` instance itself, it will be
        converted to a dictionary, containing all its current settings values
        as they would be returned by :meth:`~os_tornado.settings.Settings.get`,
        and losing all information about mutability.

        :param name: the setting name
        :type name: string

        :param default: the value to return if no setting is found
        :type default: any
        """
        value = self.get(name, default or {})
        if isinstance(value, six.string_types):
            value = json.loads(value)
        return dict(value)

    def get_float(self, name, default=0.0):
        """
        Get a setting value as a float.

        :param name: the setting name
        :type name: string

        :param default: the value to return if no setting is found
        :type default: any
        """
        return float(self.get(name, default))

    def set(self, name, value):
        self[name] = value

    def update_from_module(self, settings_module_path):
        self._assert_mutability()
        module = settings_module_path
        if isinstance(settings_module_path, six.string_types):
            module = import_module(settings_module_path)
        for key in dir(module):
            if key.isupper():
                self[key] = getattr(module, key)

    def update(self, values):
        self._assert_mutability()
        if isinstance(values, six.string_types):
            values = json.loads(values)
        if values is not None:
            for name, value in six.iteritems(values):
                self[name] = value

    def freeze(self):
        """
        Disable further changes to the current settings.

        After calling this method, the present state of the settings will become
        immutable. Trying to change values through the :meth:`~set` method and
        its variants won't be possible and will be alerted.
        """
        self.frozen = True


def iter_default_settings():
    """Return the default settings as an iterator of (name, value) tuples"""
    for name in dir(default_settings):
        if name.isupper():
            yield name, getattr(default_settings, name)


def iter_overridden_settings(settings):
    """Iterate settings that have been overridden default settings"""
    for name, def_value in iter_default_settings():
        if name not in settings:
            continue
        value = settings[name]
        if not isinstance(def_value, dict) and value != def_value:
            yield name, value


def _get_setttings_by_prefix(settings, prefix):
    p_settings = {}
    prefix_length = len(prefix)
    for key in settings:
        if key.startswith(prefix):
            k = key[prefix_length:].lower()
            p_settings[k] = settings[key]
    return p_settings


def get_tornado_app_settings(settings):
    return _get_setttings_by_prefix(settings, TORNADO_APP_SETTINGS_PREFIX)


def get_tornado_server_settings(settings):
    return _get_setttings_by_prefix(settings, TORNADO_SERVER_SETTINGS_PREFIX)



from __future__ import print_function
import re
import sys
import os
from os.path import join, exists, abspath
from shutil import copy2, ignore_patterns, copystat
import os_tornado
from os_tornado.commands import Command
from os_tornado.exceptions import UsageError

IGNORE = ignore_patterns('*.pyc', '.svn')


class C(Command):
    """Command for creating new project"""

    def syntax(self):
        return "<project_name> [project_dir]"

    def short_desc(self):
        return "Create new project."

    def _is_valid_name(self, project_name):
        if not re.search(r'^[_a-zA-Z]\w*$', project_name):
            print('Error: Project names must begin with a letter and contain'
                  ' only\nletters, numbers and underscores')
            return False
        return True

    def _copy_tpl(self, src, dst):
        ignore = IGNORE
        names = os.listdir(src)
        ignored_names = ignore(src, names)

        if not os.path.exists(dst):
            os.makedirs(dst)

        for name in names:
            if name in ignored_names:
                continue

            src_name = os.path.join(src, name)
            dst_name = os.path.join(dst, name)
            if dst_name.endswith('.tpl'):
                dst_name = dst_name[:-4]
            if os.path.isdir(src_name):
                self._copy_tpl(src_name, dst_name)
            else:
                copy2(src_name, dst_name)
        copystat(src, dst)

    def run(self, args, opts):
        if len(args) not in (1, 2):
            raise UsageError()

        project_name = args[0]
        if not self._is_valid_name(project_name):
            sys.exit(1)
        project_dir = '.'

        if len(args) == 2:
            project_dir = args[1]

        project_path = join(abspath(project_dir), project_name)
        if exists(project_path):
            print('Error: %s already exists' % project_path)
            sys.exit(1)
        self._copy_tpl(self.templates_dir, project_path)
        print("New os-tornado project: %r" % project_name)
        print("Using template directory:")
        print("    %s\n" % self.templates_dir)
        print("Create in:")
        print("    %s\n" % project_path)
        print("You can start your server with:")
        print("    cd %s" % project_path)
        print("    python manager.py runserver")

    @property
    def templates_dir(self):
        _templates_base_dir = self.settings['TEMPLATES_DIR'] or \
            join(os_tornado.__path__[0], 'commands/project_template')
        return _templates_base_dir



import inspect
import os
import sys
from importlib import import_module
from pkgutil import iter_modules


def walk_modules(module_path):
    """Loads a module and all its submodules from the given module path and
    returns them. If *any* module throws an exception while importing, that
    exception is thrown back.

    For example: walk_modules('os_tornado.utils')
    """

    mods = []
    mod = import_module(module_path)
    mods.append(mod)
    if hasattr(mod, '__path__'):
        for _, subpath, ispkg in iter_modules(mod.__path__):
            fullpath = '.'.join((module_path, subpath))
            if ispkg:
                mods += walk_modules(fullpath)
            else:
                submod = import_module(fullpath)
                mods.append(submod)
    return mods


def load_class(class_path, base_class, include_base_class=False):
    module_path, class_name = class_path.rsplit('.', 1)
    _mod = import_module(module_path)
    _cls = getattr(_mod, class_name)
    if inspect.isclass(_cls) and \
            issubclass(_cls, base_class) and \
            (include_base_class or _cls != base_class):
        return _cls
    return None


def iter_classes(module_path, base_class, include_base_class=False):
    for module in walk_modules(module_path):
        for obj in vars(module).values():
            if inspect.isclass(obj) and \
                    issubclass(obj, base_class) and \
                    obj.__module__ == module.__name__ and \
                    (include_base_class or obj != base_class):
                yield obj



import os
import pytest


def test_iter_class():
    from os_tornado.utils.module_utils import iter_classes
    from tests.test_module_utils.test_mod.class_b import BaseClass
    classes = [c for c in iter_classes('tests.test_module_utils', BaseClass)]
    expected = [
        'ClassB',
    ]
    assert set([c.__name__ for c in classes]) == set(expected)


def test_walk_modules():
    from os_tornado.utils.module_utils import walk_modules
    mods = walk_modules('tests.test_module_utils')
    expected = [
        'tests.test_module_utils',
        'tests.test_module_utils.test_mod',
        'tests.test_module_utils.test_mod.class_b',
        'tests.test_module_utils.test_mod.class_a',
        'tests.test_module_utils.test_mod0',
        'tests.test_module_utils.test_mod0.test_mod1',
        'tests.test_module_utils.test_module_utils',
    ]
    assert set([m.__name__ for m in mods]) == set(expected)


def test_load_class():
    from os_tornado.utils.module_utils import load_class
    from os_tornado.commands import Command
    from os_tornado.commands.version import VersionCommand
    obj = load_class('os_tornado.commands.version.VersionCommand', Command)
    assert obj is VersionCommand
    obj = load_class('os_tornado.commands.Command', Command)
    assert obj is None
    obj = load_class(
        'os_tornado.commands.version.VersionCommand', VersionCommand)
    with pytest.raises(ImportError):
        load_class('not_exist.NotExist', Command)
    with pytest.raises(AttributeError):
        obj = load_class('os_tornado.commands.version.NotExist', Command)
